# Helpers for installing the utility in Zope < 2.10
from zope.app.component.hooks import setSite, setHooks
from zope.component.interfaces import ComponentLookupError
from zope.component import getUtility
from five.intid.site import FiveIntIdsInstall, addUtility, add_intids
from plone.relations import interfaces
from plone.relations.container import Z2RelationshipContainer
from zope.app.intid.interfaces import IIntIds

class RelationsInstall(FiveIntIdsInstall):
    """A view for adding the local utility"""
    def install(self):
        # Add the intids utiity if it doesn't exist
        add_intids(self.context)
        add_relations(self.context)

    @property
    def installed(self):
        installed = False
        try:
            util = getUtility(interfaces.IComplexRelationshipContainer,
                                name='relations')
            if util is not None:
                installed = True
        except ComponentLookupError, e:
            pass
        return installed

def add_relations(context):
    addUtility(context, interfaces.IComplexRelationshipContainer,
               Z2RelationshipContainer, name='relations',
               findroot=False)
    # Set __name__ to the silly name given by the old component machinery:
    util = getUtility(interfaces.IComplexRelationshipContainer,
                      name='relations', context=context)
    util.__name__ = interfaces.IComplexRelationshipContainer.getName() + \
                    '-relations'
    setSite(context)
    setHooks()
    intids = getUtility(IIntIds)



from zope.interface import implementer
from zope.component import adapter, getUtility
from zope.app.intid.interfaces import IIntIds
from plone.relations.interfaces import IRelatableProxy, IRelatableUnProxy
from Products.CMFCore.utils import getToolByName
from Products.PluggableAuthService.interfaces.authservice import IBasicUser
from Products.CMFCore.interfaces import IMemberData
from zope.annotation.interfaces import IAnnotations
from zope.app.component.hooks import getSite

USER_ANNOTATION = "plone.app.relations.memberdata_annotation"

@implementer(IRelatableProxy)
@adapter(IBasicUser)
def getUserRelatableProxy(user):
    mtool = getToolByName(user, 'portal_membership')
    memberdata = mtool.getMemberById(user.getId())
    intids = getUtility(IIntIds)
    intid = intids.register(memberdata)
    annotation = IAnnotations(memberdata)
    userid = annotation.setdefault(USER_ANNOTATION, user.getId())
    return memberdata

@implementer(IRelatableUnProxy)
@adapter(IMemberData)
def getUserFromProxy(proxy):
    acl_user = getToolByName(getSite(), 'acl_users')
    annotation = IAnnotations(proxy)
    userid = annotation.get(USER_ANNOTATION)
    user = acl_user.getUserById(userid)    
    return user






from Products.GenericSetup import EXTENSION, profile_registry
from Products.CMFCore.interfaces import ISiteRoot

from Products.PluggableAuthService import registerMultiPlugin
from Products.CMFCore.permissions import ManagePortal
import local_role

registerMultiPlugin(local_role.RelationshipLocalRoleManager.meta_type)

def initialize(context):
    """Intializer called when used as a Zope 2 product."""

    # Register our PAS plugin
    context.registerClass(local_role.RelationshipLocalRoleManager,
                          permission = ManagePortal,
                          constructors = (local_role.manage_addRelationshipLocalRoleManagerForm,
                                          local_role.manage_addRelationshipLocalRoleManager),
                          visibility = None)

    profile_registry.registerProfile('default',
                                     'plone.app.relations',
                                     'Extension profile for plone.app.relatons',
                                     'profiles/default',
                                     'plone.app.relations',
                                     EXTENSION,
                                     for_=ISiteRoot)

# 2.9 alias for zope.annotation and zope.location
try:
    import zope.annotation
except ImportError:
    import sys
    import zope.app.annotation
    sys.modules['zope.annotation'] = zope.app.annotation
try:
    import zope.location
except ImportError:
    import sys
    import zope.app.location
    sys.modules['zope.location'] = zope.app.location



from plone.relations.interfaces import IComplexRelationshipContainer, _marker
from plone.relations.lazylist import lazyresolver
from plone.app.relations import interfaces
from zope.component import adapts, getUtility
from zope.interface import implements, Interface

class RelationshipTarget(object):
    """A basic implementation of IRelationshipSource based on the container
    from plone.relations, this package registers it as a named utility
    called ``relations``"""
    implements(interfaces.IRelationshipTarget)
    adapts(Interface)
    _name = 'relations'

    def __init__(self, target):
        self.target = target
        # always use the context of the target object for utility lookup
        self.util = getUtility(IComplexRelationshipContainer, name=self._name)
        self._resolver = self.util.relationIndex.resolveValueTokens

    def _target_resolver(self, value):
        return self._resolver((value,), 'target').next()

    def getRelationships(self, source=None, relation=_marker, state=_marker,
                         context=_marker, rel_filter=None):
        """See interface"""
        rels = self.util.findRelationships(source, self.target, relation,
                                           state, context, filter=rel_filter)
        for chain in rels:
            yield chain[0]

    def isLinked(self, source=None, relation=_marker, state=_marker,
                 context=_marker, rel_filter=None, maxDepth=1,
                 minDepth=None, transitivity=None):
        """See interface"""
        return self.util.isLinked(source, self.target, relation, state,
                                  context, maxDepth, minDepth,
                                  filter=rel_filter, transitivity=transitivity)

    def getRelationshipChains(self, source=None, relation=_marker,
                               state=_marker, context=_marker, rel_filter=None,
                               maxDepth=1, minDepth=None, transitivity=None):
        """See interface"""
        return self.util.findRelationships(source, self.target, relation,
                                           state, context, maxDepth, minDepth,
                                           filter=rel_filter,
                                           transitivity=transitivity)

    @lazyresolver(resolver_name='_target_resolver')
    def getSources(self, relation=_marker, state=_marker, context=_marker,
                    rel_filter=None, maxDepth=1, minDepth=None,
                    transitivity=None):
        """See interface"""
        return self.util.findSourceTokens(self.target, relation, state,
                                     context, maxDepth, minDepth,
                                     filter=rel_filter,
                                     transitivity=transitivity)


class SymmetricRelation(object):
    implements(interfaces.ISymmetricRelation)
    adapts(Interface)
    _name = 'relations'

    def __init__(self, obj):
        self.obj = obj
        # always use the context of the object for utility lookup
        self.util = getUtility(IComplexRelationshipContainer, name=self._name)
        self._resolver = self.util.relationIndex.resolveValueTokens
        self._rel_resolver = self.util.relationIndex.resolveRelationshipTokens

    def _relation_resolver(self, entry):
        value, t_type = entry
        return self._resolver((value,), t_type).next()

    def _relationship_resolver(self, value):
        return self._rel_resolver((value,)).next()

    def _getRelationshipTokens(self, partner=None, relation=_marker,
                               state=_marker, context=_marker, rel_filter=None):
        """See interface"""
        rels1 = self.util.findRelationshipTokens(self.obj, partner, relation,
                                                 state, context,
                                                 filter=rel_filter)
        rels2 = self.util.findRelationshipTokens(partner, self.obj, relation,
                                                 state, context,
                                                 filter=rel_filter)
        seen = []
        for chain in rels1:
            token = chain[0]
            seen.append(token)
            yield token
        for chain in rels2:
            token = chain[0]
            if token not in seen:
                yield token

    @lazyresolver(resolver_name='_relationship_resolver')
    def getRelationships(self, partner=None, relation=_marker, state=_marker,
                        context=_marker, rel_filter=None):
        rels = self._getRelationshipTokens(partner, relation, state, context,
                                           rel_filter)
        return rels

    def _getRelationTokens(self, relation=_marker, state=_marker,
                           context=_marker, rel_filter=None):
        """See interface"""
        targets = self.util.findTargetTokens(self.obj, relation, state, context,
                                             filter=rel_filter)
        sources = self.util.findSourceTokens(self.obj, relation, state, context,
                                             filter=rel_filter)
        seen = []
        for t in targets:
            seen.append(t)
            yield t, 'target'
        for t in sources:
            if t not in seen:
                yield t, 'source'

    @lazyresolver(resolver_name='_relation_resolver')
    def getRelations(self, relation=_marker, state=_marker,
                     context=_marker, rel_filter=None):
        rels = self._getRelationTokens(relation, state, context, rel_filter)
        return rels

    def isLinked(self, partner=None, relation=_marker, state=_marker,
                 context=_marker, rel_filter=None):
        as_source = self.util.isLinked(self.obj, partner, relation, state,
                                       context, filter=rel_filter)
        return as_source or self.util.isLinked(partner, self.obj, relation,
                                               state, context,
                                               filter=rel_filter)





import sys
from Acquisition import aq_base
from zope.interface import implements
from zope.component import adapts
from zope.app.component.hooks import getSite
from plone.app.relations import interfaces
from plone.app.relations.annotations import ANNOTATIONS_KEY
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.WorkflowCore import ObjectDeleted
from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
from zope.app.annotation.interfaces import IAnnotations
from persistent.mapping import PersistentMapping
from zc.relationship.interfaces import IBidirectionalRelationshipIndex

_marker = []

class DCWorkflowAdapter(object):

    """ This adapter provides the IStatefulRelationship interface
    using the DCWorkflow engine from the CMF. As a result it requires
    a fair amount of existing infrastructure, including a workflow
    tool with a relevant workflow.  Our testing infrastructure has set
    up a fully functioning plone site for us.

    To demonstrate how to use this adapter, we build some stateful
    relationships between our site content:

        >>> from plone.app.relations import tests
        >>> tests.base_setup(portal)
        >>> from plone.app.relations import interfaces
        >>> from zope.app.annotation.interfaces import IAttributeAnnotatable
        >>> ob1 = portal['ob1']
        >>> ob2 = portal['ob2']
        >>> ob3 = portal['ob3']
        >>> source = interfaces.IRelationshipSource(ob1)
        >>> rel = source.createRelationship(ob2, relation=u'relation 1',
        ...             interfaces=(interfaces.IDCWorkflowableRelationship,
        ...                         IAttributeAnnotatable))


    Let's ghost this object before retrieving it again::

        >>> import transaction
        >>> sp = transaction.savepoint()
        >>> rel._p_deactivate()
        >>> rel.targets._p_deactivate()
        >>> rel.sources._p_deactivate()
        >>> rel = list(source.getRelationships())[0]

    We can see that this relationship currently has no state::

        >>> list(source.getRelationships(state=None))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]
        >>> list(source.getRelationships(state='visible'))
        []

    Now we have our workflowable relationship, let's specify a workflow for it.
    We'll use the ``plone_workflow`` for convenience, though it is not
    necessarily a sensible workflow for a relationship.

        >>> stateful = interfaces.IStatefulRelationship(rel)
        >>> stateful.workflow_id = 'plone_workflow'

    Now our relationship should be in the default state of this
    workflow ``visible``, ad should be searchable based on this state::

        >>> stateful.state
        'visible'
        >>> list(source.getRelationships(state=None))
        []
        >>> list(source.getRelationships(state='visible'))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]

    We can now see what transitions are available for the Owner in this
    relationship's state, and of course execute them::

        >>> self.setRoles(['Owner', 'Member'])
        >>> transitions = stateful.listActions()
        >>> len(transitions)
        2
        >>> transitions[0]['id']
        'hide'
        >>> transitions[1]['id']
        'submit'
        >>> stateful.doAction('submit', 'A comment for my submission')
        >>> stateful.state
        'pending'

    We should now be able to find our relationship using the new
    ``pending`` state::

        >>> list(source.getRelationships(state='visible'))
        []
        >>> list(source.getRelationships(state='pending'))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]

    We should also be able to query our relationship for workflow variables
    besides state::

        >>> stateful.getInfo('review_history')
        [{'action': None, 'review_state': 'visible', 'actor': 'test_user_1_', 'comments': '', 'time': DateTime(...)}, {'action': 'submit', 'review_state': 'pending', 'actor': 'test_user_1_', 'comments': 'A comment for my submission', 'time': DateTime(...)}]

    We can also check if a particular action is allowed at the moment:

        >>> stateful.isActionAllowed('retract')
        True
        >>> stateful.isActionAllowed('publish')
        False

    If we change the workflow used, we get shifted to the default
    state of that workflow, and the relationship is reindexed::

        >>> stateful.workflow_id = 'folder_workflow'
        >>> stateful.state
        'visible'
        >>> list(source.getRelationships(state='visible'))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]
        >>> stateful.workflow_id = 'plone_workflow'
        >>> stateful.state
        'visible'
        >>> list(source.getRelationships(state='visible'))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]

    We can also set the name of the state variable for the workflow, which
    is helpful when using custom workflows that don't use the standard
    ``review_state``.  Here we set it to ``actor`` to demonstrate::

        >>> stateful.state_var = 'actor'
        >>> stateful.state
        'test_user_1_'
        >>> list(source.getRelationships(state='test_user_1_'))
        [<Relationship u'relation 1' from (<Demo ob1>,) to (<Demo ob2>,)>]

    We should also test the behavior for some pathological cases, which
    we expect to raise WorkflowException, though ``state`` should always
    return something sane::

        >>> stateful.doAction('bogus_action')
        Traceback (most recent call last):
        ...
        WorkflowException: No workflow provides the "bogus_action" action.
        >>> stateful.workflow_id = None
        >>> stateful.getInfo('review_state')
        Traceback (most recent call last):
        ...
        WorkflowException: Workflow definition not yet set.
        >>> stateful.workflow_id = 'bogus_workflow'
        Traceback (most recent call last):
        ...
        WorkflowException: Reqested workflow definition not found.
        >>> stateful.state is None
        True
    """

    implements(interfaces.IDCWorkflowRelationship)
    adapts(interfaces.IDCWorkflowableRelationship)

    def __init__(self, rel):
        # XXX: wrap our relationship in the current Site
        site_chain = getSite().aq_chain
        site_chain.insert(0, rel)
        rel = site_chain.pop(-1)
        for item in reversed(site_chain):
            rel = aq_base(site_chain.pop(-1)).__of__(rel)
        self.rel = rel
        # Use annotations for storing state
        self.annotations = IAnnotations(rel).setdefault(ANNOTATIONS_KEY,
                                                        PersistentMapping())
        # use the current Site for the tool lookup
        self.wf_tool = getToolByName(getSite(), 'portal_workflow')

    @apply
    def state_var():
        def get(self):
            return self.annotations.get('wf_state_var', 'review_state')
        def set(self, value):
            self.annotations['wf_state_var'] = value
            # reindex for new state
            if IBidirectionalRelationshipIndex.providedBy(self.rel.__parent__):
                self.rel.__parent__.reindex(self.rel)
        return property(get, set)

    @apply
    def workflow_id():
        def get(self):
            return self.annotations.get('dcworkflow_id', None)
        def set(self, value):
            self.annotations['dcworkflow_id'] = value
            if value is not None:
                wf = self._get_workflow()
                # reset workflow state to intial state of new workflow
                # XXX: Should we try to use an existing state when we switch back
                # to a workflow we had before?
                wf.notifyCreated(self.rel)
            # reindex for new state
            if IBidirectionalRelationshipIndex.providedBy(self.rel.__parent__):
                self.rel.__parent__.reindex(self.rel)
        return property(get, set)

    @property
    def state(self):
        # XXX: No acquisition path here
        try:
            return self.getInfo(self.state_var, None)
        except WorkflowException:
            return None

    def doAction(self, action, comment='', **kw):
        wf = self._get_workflow()
        if not wf.isActionSupported(self.rel, action, **kw):
            raise WorkflowException(
                'No workflow provides the "%s" action.' % action)
        wf.notifyBefore(self.rel, action)
        # notify(Blah)

        # XXX: no support for moving relationships during workflow for now
        try:
            res = wf.doActionFor(self.rel, action, comment, **kw)
        except ObjectDeleted, ex:
            res = ex.getResult()
        except:
            exc = sys.exc_info()
            wf.notifyException(self.rel, action, exc)
            #notify(Blah)
            raise
        wf.notifySuccess(self.rel, action, res)
        # Reindex (this should probably be triggered by an event)
        if IBidirectionalRelationshipIndex.providedBy(self.rel.__parent__):
            self.rel.__parent__.reindex(self.rel)
        #notify(Blah)
        return res

    def isActionAllowed(self, action):
        wf = self._get_workflow()
        return bool(wf.isActionSupported(self.rel, action))

    def getInfo(self, name, default=_marker, *args, **kw):
        wf = self._get_workflow()
        res = wf.getInfoFor(self.rel, name, default, *args, **kw)
        if res is _marker:
            raise WorkflowException('Could not get info: %s' % name)
        return res

    def listActions(self):
        result = {}
        wf = self._get_workflow()
        sdef = wf._getWorkflowStateOf(self.rel)
        # The url to the relationship should never be used we just use a name
        obj_url = '/'.join(self.rel.__name__)
        if sdef is not None:
            for tid in sdef.transitions:
                tdef = wf.transitions.get(tid, None)
                if tdef is not None and \
                   tdef.trigger_type == TRIGGER_USER_ACTION and \
                   wf._checkTransitionGuard(tdef, self.rel) and \
                   not result.has_key(tdef.id):
                    result[tdef.id] = {
                        'id': tdef.id,
                        'title': tdef.title,
                        'title_or_id': tdef.title_or_id(),
                        'description': tdef.description,
                        'name': tdef.actbox_name or tdef.title_or_id(),
                        'url': tdef.actbox_url %
                        {'content_url': obj_url,
                         'portal_url' : '',
                         'folder_url' : ''}}
        return tuple(result.values())

    def _get_workflow(self):
        if not self.workflow_id:
            raise WorkflowException('Workflow definition not yet set.')
        # XXX: Add Plone 3.0 compat
        wf = self.wf_tool.getWorkflowById(self.workflow_id)
        if wf is None:
            raise WorkflowException(
                'Reqested workflow definition not found.')
        return wf



from zope.interface import Interface, Attribute, implements
from zope.annotation.interfaces import IAnnotatable
try:
    from zope.component.interfaces import ObjectEvent
    from zope.component.interfaces import IObjectEvent
except ImportError:
    from zope.app.event.objectevent import ObjectEvent
    from zope.app.event.interfaces import IObjectEvent
from plone.relations.interfaces import _marker, IStatefulRelationship

class TooManyResultsError(Exception):
    """Indicates that a query which was expected to generate a single result
    returned multiple."""

class NoResultsError(Exception):
    """Indicates that a query which was expected to generate results did not."""

class IRelationshipSource(Interface):
    """A content-centric API for a plone.relations based utility"""

    def createRelationship(targets, relation=None, interfaces=(),
                           rel_factory=None):
        """Creates a relationship between the current object (the
        source) and any number of target objects.  Detects if targets
        is a single object or a sequence.  The relation is an optional
        string to describe the type of relationship involved.

        The ``interfaces`` argument allows passing in a sequence of
        marker interfaces to be applied directly to the created
        relationship instance.  This is the preferred mechanism for
        applying custom behavior to a particular relationship.

        The rel_factory allows passing in an optional factory to be
        used for creating the relationship instances.  A sane default
        relationship class will be provided.  This mechanism is
        discouraged for all situations in which ``interfaces`` is a
        viable option, and should only be used when custom behavior
        is essential to have directly in the instance (e.g. required
        Zope 2 mixins like Role Manager when a relationship needs
        per instance security configuration).

        This method returns the relationship object in case further
        configuration is needed.  Such configuration should generally
        be done via adaptation."""

    def getRelationships(target=None, relation=_marker, state=_marker,
                           context=_marker, rel_filter=None):
        """Finds relationships to the object matching the specified
        parameters, targets, relation, state, context.  If no
        parameters are provided, then all relations to the adapted
        object will be found.  Optionally only returns relationships
        for which pass the rel_filter returns True."""

    def deleteRelationship(target=None, relation=_marker, state=_marker,
                           context=_marker, rel_filter=None, multiple=False,
                           remove_all_targets=False, ignore_missing=False):
        """Removes a relationship matching the specified parameters,
        ``target``, ``relation``, ``state``, ``context``.  Optionally
        only returns relationships which pass the ``rel_filter``
        (e.g. IMyInterface.providedBy).  If the query returns more
        than one relationship then ``TooManyResultsError`` will be
        raised unless the parameter ``multiple`` is set.  If the found
        relationship(s) have multiple targets, then the specified
        target is removed from the relationship unless
        ``remove_all_targets`` is set, in which case the matching
        relation will be removed entirely.  If no query parameters are
        specified and ``multiple`` is True, then all relationships to
        the object will be removed.  Not specifying a ``target`` is
        equivalent to turning on ``remove_all_targets``.

        Note: ``remove_all_targets`` is potentially quite dangerous.
        It is important that the method sanely handles the case where
        the retrieved relationship has multiple sources.  In this
        case, if there is only one target or if ``remove_all_targets``
        is True, the specified source should be removed from the
        relationship.  Otherwise, a ``TooManyResultsError`` may be
        raised, with a message indicating the problem.

        Will raise ``NoResultsError`` if no matching relationships are
        found unless ``ignore_missing`` is specified.
        """

    def isLinked(target=None, relation=_marker, state=_marker,
                 context=_marker, rel_filter=None, maxDepth=1,
                 minDepth=None, transitivity=None):
        """Returns True if a relationship exists matching the given
        parameters.  You can pass an optional maxDepth (None => no
        limit) and minDepth so that relationships are transitively
        determined.  A custom transitivity factory can be passed in,
        see the zc.relationship docs for more information."""

    def getRelationshipChains(target=None, relation=_marker,
                               state=_marker, context=_marker, rel_filter=None,
                               maxDepth=1, minDepth=None, transitivity=None):
        """Returns a sequence of tuples of relationships each
        representing a chain of relationships which match the given
        parameters and lead to the specified target.  Depth and means of
        transitivity can be specified as with isLinked."""

    def getTargets(relation=_marker, state=_marker, context=_marker,
                    rel_filter=None, maxDepth=1, minDepth=None,
                    transitivity=None):
        """Returns a list of all target objects whose relationships
        match the specified parameters.  Depth and means of
        transitivity can be specified as with isLinked."""


class IRelationshipTarget(Interface):
    """A content-centric api for looking at back references"""

    def getRelationships(source=None, relation=_marker, state=_marker,
                           context=_marker, rel_filter=None):
        """Finds relationships from the object matching the specified
        parameters.  See IRelationshipSource for more details."""

    def isLinked(source=None, relation=_marker, state=_marker,
                 context=_marker, rel_filter=None, maxDepth=1,
                 minDepth=None, transitivity=None):
        """Returns True if a relationship exists matching the given
        parameters.  See IRelationshipSource for more details."""

    def getRelationshipChains(source=None, relation=_marker,
                               state=_marker, context=_marker, rel_filter=None,
                               maxDepth=1, minDepth=None, transitivity=None):
        """Returns a sequence of tuples of relationships each
        representing a chain of relationships which match the given
        parameters and lead to the specified target.  Depth and means of
        transitivity can be specified as with isLinked."""

    def getSources(relation=_marker, state=_marker, context=_marker,
                    rel_filter=None, maxDepth=1, minDepth=None,
                    transitivity=None):
        """Returns a list of all related source objects whose
        relatioships match the specified parameters.  Depth and means
        of transitivity can be specified as with isLinked."""


class ISymmetricRelation(Interface):
    """Allows searching for relationships and related objects
    regardless of whether the current object is the source or the
    target of the relationship."""

    def getRelationships(partner=None, relation=_marker, state=_marker,
                         context=_marker, rel_filter=None):
        """Returns all relationships pointing to or from the current object,
        matching the given parameters.  You may optionally specify a
        partner which is another object involved in the relationship."""

    def getRelations(relation=_marker, state=_marker, context=_marker,
                     rel_filter=None):
        """Returns all relationships pointing to or from the current object,
        matching the given parameters.  You may optionally specify a
        partner which is another object involved in the relationship."""

    def isLinked(partner=None, relation=_marker, state=_marker,
                           context=_marker, rel_filter=None):
        """Returns true if the current object is linked
        (non-transitively) to the partner object as source or target
        (with restrictions provided by the other parameters).  If no
        partner is specified, returns true if there are any links to
        or from the object matching the given parameters."""


# Marker interfaces for determinig application of custom behavior

class IAnnotationsState(IAnnotatable):
    """Implies that the relationship state should be stored in an annotation"""

class IAnnotationsContext(IAnnotatable):
    """Implies that the relationship context should be stored in an
    annotation"""

class IDCWorkflowableRelationship(IAnnotatable):
    """A marker that indicates that state information on the object
    will be managed using DCWorkflow"""

class IDCWorkflowRelationship(IStatefulRelationship):
    """An interface describing a stateful relationship whose state is
    manipulated using a DCWorkflow (from CMF)"""

    workflow_id = Attribute('The name of a DC workflow to be used for this '
                               'object')
    state_var = Attribute('The name of the wf variable in which the workflow '
                          'state is stored.')

    def doAction(action, comment='', **kw):
        """Requests a workflow action on the object"""

    def isActionAllowed(action):
        """Returns True if the given action is currently allowed"""

    def getInfo(name, default=_marker):
        """Returns the named workflow variable after checking if the current
        user is allowed to access it"""

    def listActions():
        """List available user triggered transitions"""

class IRelationSourceDeleted(IObjectEvent):
    """An event which indicates that the source of a relationship has
    been deleted"""
    source = Attribute("The object that was deleted")

class IRelationTargetDeleted(IObjectEvent):
    """An event which indicates that the target of a relationship has
    been deleted"""
    target = Attribute("The object that was deleted")

class RelationSourceDeleted(ObjectEvent):
    implements(IRelationSourceDeleted)

    def __init__(self, object, source):
        ObjectEvent.__init__(self, object)
        self.source = source

class RelationTargetDeleted(ObjectEvent):
    implements(IRelationTargetDeleted)

    def __init__(self, object, target):
        ObjectEvent.__init__(self, object)
        self.target = target

class IDefaultDeletion(Interface):
    """A marker interface indicating that the relationship should be
    deleted/updated when it's sources or targets are deleted"""

class IHoldingRelation(Interface):
    """A marker interface to demonstrate the use of a marker interface
    to change behavior on target deletion"""

class HoldingRelationError(Exception):
    """An exception raised when the target of a HoldingRelation is deleted"""
    def __init__(self, obj, relation):
        self.args =  "%s cannot be deleted, it is the target of a "\
                    "relationship to %s"%(obj, tuple(relation.sources))

class IRetainOnCopy(Interface):
    """A marker interface to indicate that a relationship should be copied
    when it's source object is copied (with the sources changed on the copy)."""

class ICopyPendingAdd(Interface):
    """An interface indicating that an object is a copy that hasn't been added
    yet.  This is used to filter for add events that result from a copy.
    The interface promises an attribute pointing to the original object"""

    __orig_object__ = Attribute("The original object before copy")

class ILocalRoleProvider(Interface):
    """An interface which allows querying the local roles on an object"""

    def getRoles(principal_id):
        """Returns an iterable of roles granted to the specified user object"""

    def getAllRoles():
        """Returns an iterable consisting of tuples of the form:
                    (principal_id, sequence_of_roles)
        """
class IFactoryTempFolder(Interface):
    """A marker indicating the portal_factory temp folder"""



from setuptools import setup, find_packages

version = '1.0b2'

setup(name='plone.app.relations',
      version=version,
      description="A set of components to provide a content centric API for the engine from plone.relations, as well as a few different core relationship types and policies.",
      long_description="""\
""",
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='relationships references plone',
      author='Alec Mitchell',
      author_email='apm13@columbia.edu',
      url='http://svn.plone.org/svn/plone/plone.app.relations',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          "plone.relations",
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import unittest
from zope.app.testing import placelesssetup
from zope.interface import implements
from zope.testing.doctest import DocTestSuite
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc
from OFS.SimpleItem import SimpleItem

from five.intid.lsm import USE_LSM
from five.intid.site import add_intids
from plone.app.relations.utils import add_relations
from plone.app.relations.interfaces import ILocalRoleProvider

from Products.Five import zcml


class Demo(SimpleItem):
    def __init__(self, id):
        self.id = id
    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, self.id)

def contentSetUp(app):
    for i in range(30):
        oid = 'ob%d' % i
        app._setObject(oid, Demo(oid))


class SimpleLocalRoleProvider(object):
    implements(ILocalRoleProvider)
    def __init__(self, context):
        self.context = context

    def getRoles(self, user):
        """Grant everyone the 'Foo' role"""
        return ('Foo',)

    def getAllRoles(self):
        """In the real world we would enumerate all users and
        grant the 'Foo' role to each, but we won't"""
        yield ('bogus_user', ('Foo',))


class DummyUser(object):
    def __init__(self, uid, group_ids=()):
        self.id = uid
        self._groups = group_ids

    def getId(self):
        return self.id

    def _check_context(self, obj):
        return True

    def getGroups(self):
        return self._groups


def base_setup(app):
    """Setup without basic CA stuff because PTC already provides this"""
    from plone.app import relations
    try:
        zcml.load_config('configure.zcml', relations)
    except:
        # XXX: When ptc has setup plone, then the zcml product
        # registration causes errors when run twice :-(
        pass
    if not USE_LSM:
        # monkey in our hooks
        from Products.Five.site.metaconfigure import classSiteHook
        from Products.Five.site.localsite import FiveSite
        from zope.interface import classImplements
        from zope.app.component.interfaces import IPossibleSite
        klass = app.__class__
        classSiteHook(klass, FiveSite)
        classImplements(klass, IPossibleSite)
    add_intids(app)
    add_relations(app)
    contentSetUp(app)

def setUp(app):
    """A setup that includes the basic CA setup"""
    placelesssetup.setUp()
    import Products.Five
    zcml.load_config('meta.zcml', Products.Five)
    zcml.load_config('permissions.zcml', Products.Five)
    zcml.load_config('configure.zcml', Products.Five)
    base_setup(app)


def tearDown():
    placelesssetup.tearDown()

# XXX: This messes things up, where else could we call it, it's only needed
# for workflow
ptc.setupPloneSite()

def test_suite():
    from plone.app.relations import local_role
    pas = DocTestSuite(local_role, setUp=placelesssetup.setUp(),
                       tearDown=placelesssetup.tearDown())

    readme = ztc.FunctionalDocFileSuite('README.txt',
                                        package='plone.app.relations')

    workflow = ztc.ZopeDocTestSuite('plone.app.relations.workflow',
                                    test_class=ptc.FunctionalTestCase,)

    userrelations = ztc.ZopeDocFileSuite('userrelations.txt',
                                         package='plone.app.relations', 
                                         test_class=ptc.FunctionalTestCase,)


    return unittest.TestSuite([workflow, readme, pas, userrelations])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
